/** 
 * DevExtreme (localization/globalize/message.js)
 * Version: 16.1.4
 * Build date: Wed Jun 22 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web, DevExtreme Mobile
 */
"use strict";
define(function(require, exports, module) {
    require("./core");
    var $ = require("jquery"),
        stringFormat = require("../../core/utils/string").format,
        Globalize = require("globalize"),
        messageLocalization = require("../message");
    require("globalize/message");
    if (!Globalize || !Globalize.formatMessage) {
        return
    }
    var messagesCache = {};
    var globalizeMessageLocalization = {
        ctor: function() {
            this.load(this._dictionary)
        },
        load: function(messages) {
            $.extend(true, messagesCache, messages);
            Globalize.loadMessages(messages)
        },
        locale: function(locale) {
            if (!locale) {
                return Globalize.locale().locale
            }
            return Globalize.locale().locale
        },
        _messageLoaded: function(key, locale) {
            var currentCldr = locale ? new Globalize(locale).cldr : Globalize.locale(),
                value = currentCldr.get(["globalize-messages/{bundle}", key]);
            return void 0 !== value
        },
        getMessagesByLocales: function() {
            return Globalize.cldr.get("globalize-messages")
        },
        getFormatter: function(path) {
            var locale = Globalize.locale().locale,
                localeMessages = messagesCache[locale],
                message = localeMessages && localeMessages[path];
            if (message) {
                return function() {
                    var args = 1 === arguments.length && Array.isArray(arguments[0]) ? arguments[0].slice(0) : Array.prototype.slice.call(arguments, 0);
                    args.unshift(message);
                    return stringFormat.apply(this, args)
                }
            }
            return Globalize.messageFormatter(path)
        },
        format: function(key, value, locale) {
            var currentLocale = Globalize.locale().locale,
                currentGlobalize = !locale || locale === currentLocale ? Globalize : new Globalize(locale),
                localeMessages = messagesCache[locale || currentLocale],
                result = localeMessages && localeMessages[key];
            if (void 0 !== result) {
                if (void 0 !== value) {
                    result = stringFormat(result, value)
                }
                return result
            }
            try {
                result = currentGlobalize.formatMessage(key, value)
            } catch (e) {}
            return result || ""
        }
    };
    messageLocalization.inject(globalizeMessageLocalization)
});
