/** 
 * DevExtreme (ui/tooltip/tooltip.js)
 * Version: 16.1.4
 * Build date: Wed Jun 22 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web, DevExtreme Mobile
 */
"use strict";
define(function(require, exports, module) {
    var $ = require("jquery"),
        Guid = require("../../core/guid"),
        registerComponent = require("../../core/component_registrator"),
        pointerEvents = require("../../events/pointer"),
        Popover = require("../popover"),
        TOOLTIP_CLASS = "dx-tooltip",
        TOOLTIP_WRAPPER_CLASS = "dx-tooltip-wrapper",
        TOOLTIP_STACK = [],
        FIRST_Z_INDEX_OFFSET = -500;
    $(document).on(pointerEvents.down, function(e) {
        for (var i = TOOLTIP_STACK.length - 1; i >= 0; i--) {
            if (!TOOLTIP_STACK[i]._proxiedDocumentDownHandler(e)) {
                return
            }
        }
    });
    var Tooltip = Popover.inherit({
        _getDefaultOptions: function() {
            return $.extend(this.callBase(), {
                toolbarItems: [],
                showCloseButton: false,
                showTitle: false,
                title: null,
                titleTemplate: null,
                onTitleRendered: null,
                bottomTemplate: null
            })
        },
        _render: function() {
            this.element().addClass(TOOLTIP_CLASS);
            this._wrapper().addClass(TOOLTIP_WRAPPER_CLASS);
            this.callBase()
        },
        _renderContent: function() {
            this.callBase();
            this._contentId = new Guid;
            this._$content.attr({
                id: this._contentId,
                role: "tooltip"
            });
            this._toggleAriaDescription(true)
        },
        _toggleAriaDescription: function(showing) {
            var $target = $(this.option("target")),
                label = showing ? this._contentId : void 0;
            this.setAria("describedby", label, $target)
        },
        _overlayStack: function() {
            return TOOLTIP_STACK
        },
        _zIndexInitValue: function() {
            return this.callBase() + FIRST_Z_INDEX_OFFSET
        }
    });
    registerComponent("dxTooltip", Tooltip);
    module.exports = Tooltip
});
