/** 
 * DevExtreme (viz/core/loading_indicator.js)
 * Version: 16.1.4
 * Build date: Wed Jun 22 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web
 */
"use strict";
define(function(require, exports, module) {
    var _patchFontOptions = require("./utils").patchFontOptions,
        STATE_HIDDEN = 0,
        STATE_SHOWN = 1,
        ANIMATION_EASING = "linear",
        ANIMATION_DURATION = 400,
        LOADING_INDICATOR_READY = "loadingIndicatorReady";

    function LoadingIndicator(parameters) {
        var that = this,
            renderer = parameters.renderer;
        that._group = renderer.g().attr({
            "class": "dx-loading-indicator"
        }).linkOn(renderer.root, "loading-indicator");
        that._rect = renderer.rect().attr({
            opacity: 0
        }).append(that._group);
        that._text = renderer.text().attr({
            align: "center"
        }).append(that._group);
        that._createStates(parameters.eventTrigger, that._group, renderer.root, parameters.notify)
    }
    LoadingIndicator.prototype = {
        constructor: LoadingIndicator,
        _createStates: function(eventTrigger, group, root, notify) {
            var that = this;
            that._states = [{
                opacity: 0,
                start: function() {
                    notify(false)
                },
                complete: function() {
                    group.linkRemove();
                    root.attr({
                        "pointer-events": null
                    });
                    eventTrigger(LOADING_INDICATOR_READY)
                }
            }, {
                opacity: .85,
                start: function() {
                    group.linkAppend();
                    root.attr({
                        "pointer-events": "none"
                    });
                    notify(true)
                },
                complete: function() {
                    eventTrigger(LOADING_INDICATOR_READY)
                }
            }];
            that._state = STATE_HIDDEN
        },
        setSize: function(size) {
            var width = size.width,
                height = size.height;
            this._rect.attr({
                width: width,
                height: height
            });
            this._text.attr({
                x: width / 2,
                y: height / 2
            })
        },
        setOptions: function(options) {
            this._rect.attr({
                fill: options.backgroundColor
            });
            this._text.css(_patchFontOptions(options.font)).attr({
                text: options.text
            });
            this[options.show ? "show" : "hide"]()
        },
        dispose: function() {
            var that = this;
            that._group.linkRemove().linkOff();
            that._group = that._rect = that._text = that._states = null
        },
        _transit: function(stateId) {
            var state, that = this;
            if (that._state !== stateId) {
                that._state = stateId;
                that._isHiding = false;
                state = that._states[stateId];
                that._rect.stopAnimation().animate({
                    opacity: state.opacity
                }, {
                    complete: state.complete,
                    easing: ANIMATION_EASING,
                    duration: ANIMATION_DURATION,
                    unstoppable: true
                });
                that._noHiding = true;
                state.start();
                that._noHiding = false
            }
        },
        show: function() {
            this._transit(STATE_SHOWN)
        },
        hide: function() {
            this._transit(STATE_HIDDEN)
        },
        scheduleHiding: function() {
            if (!this._noHiding) {
                this._isHiding = true
            }
        },
        fulfillHiding: function() {
            if (this._isHiding) {
                this.hide()
            }
        }
    };
    exports.LoadingIndicator = LoadingIndicator
});
