/** 
 * DevExtreme (viz/translators/range.js)
 * Version: 16.1.4
 * Build date: Wed Jun 22 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web
 */
"use strict";
define(function(require, exports, module) {
    var _Range, $ = require("jquery"),
        commonUtils = require("../../core/utils/common"),
        _isDefined = commonUtils.isDefined,
        _isDate = commonUtils.isDate,
        minSelector = "min",
        maxSelector = "max",
        minVisibleSelector = "minVisible",
        maxVisibleSelector = "maxVisible",
        categoriesSelector = "categories",
        baseSelector = "base",
        axisTypeSelector = "axisType";

    function otherLessThan(thisValue, otherValue) {
        return otherValue < thisValue
    }

    function otherGreaterThan(thisValue, otherValue) {
        return otherValue > thisValue
    }

    function compareAndReplace(thisValue, otherValue, setValue, compare) {
        var otherValueDefined = _isDefined(otherValue);
        if (_isDefined(thisValue)) {
            if (otherValueDefined && compare(thisValue, otherValue)) {
                setValue(otherValue)
            }
        } else {
            if (otherValueDefined) {
                setValue(otherValue)
            }
        }
    }

    function returnValue(value) {
        return value
    }

    function returnValueOf(value) {
        return value.valueOf()
    }

    function getUniqueCategories(categories, otherCategories, dataType) {
        var found, i, j, length = categories.length,
            valueOf = "datetime" === dataType ? returnValueOf : returnValue;
        if (otherCategories && otherCategories.length) {
            for (i = 0; i < otherCategories.length; i++) {
                for (j = 0, found = false; j < length; j++) {
                    if (valueOf(categories[j]) === valueOf(otherCategories[i])) {
                        found = true;
                        break
                    }
                }!found && categories.push(otherCategories[i])
            }
        }
        return categories
    }
    _Range = exports.Range = function(range) {
        range && $.extend(this, range)
    };
    _Range.prototype = {
        constructor: _Range,
        dispose: function() {
            this[categoriesSelector] = null
        },
        addRange: function(otherRange) {
            var that = this,
                categories = that[categoriesSelector],
                otherCategories = otherRange[categoriesSelector];
            var compareAndReplaceByField = function(field, compare) {
                compareAndReplace(that[field], otherRange[field], function(value) {
                    that[field] = value
                }, compare)
            };
            var controlValuesByVisibleBounds = function(valueField, visibleValueField, compare) {
                compareAndReplace(that[valueField], that[visibleValueField], function(value) {
                    _isDefined(that[valueField]) && (that[valueField] = value)
                }, compare)
            };
            var checkField = function(field) {
                that[field] = that[field] || otherRange[field]
            };
            if (commonUtils.isDefined(otherRange.stick)) {
                that.stick = otherRange.stick
            }
            checkField("addSpiderCategory");
            checkField("percentStick");
            checkField("minSpaceCorrection");
            checkField("maxSpaceCorrection");
            checkField("invert");
            checkField(axisTypeSelector);
            checkField("dataType");
            if ("logarithmic" === that[axisTypeSelector]) {
                checkField(baseSelector)
            } else {
                that[baseSelector] = void 0
            }
            compareAndReplaceByField(minSelector, otherLessThan);
            compareAndReplaceByField(maxSelector, otherGreaterThan);
            if ("discrete" === that[axisTypeSelector]) {
                checkField(minVisibleSelector);
                checkField(maxVisibleSelector)
            } else {
                compareAndReplaceByField(minVisibleSelector, otherLessThan);
                compareAndReplaceByField(maxVisibleSelector, otherGreaterThan)
            }
            compareAndReplaceByField("interval", otherLessThan);
            controlValuesByVisibleBounds(minSelector, minVisibleSelector, otherLessThan);
            controlValuesByVisibleBounds(minSelector, maxVisibleSelector, otherLessThan);
            controlValuesByVisibleBounds(maxSelector, maxVisibleSelector, otherGreaterThan);
            controlValuesByVisibleBounds(maxSelector, minVisibleSelector, otherGreaterThan);
            if (void 0 === categories) {
                that[categoriesSelector] = otherCategories
            } else {
                that[categoriesSelector] = getUniqueCategories(categories, otherCategories, that.dataType)
            }
            return this
        },
        isDefined: function() {
            return _isDefined(this[minSelector]) && _isDefined(this[maxSelector]) || this[categoriesSelector] && this[categoriesSelector].length
        },
        setStubData: function(dataType) {
            var that = this,
                year = (new Date).getYear() - 1,
                isDate = "datetime" === dataType,
                isCategories = "discrete" === that[axisTypeSelector];
            if (isCategories) {
                that.categories = ["0", "1", "2"]
            } else {
                that[minSelector] = isDate ? new Date(year, 0, 1) : 0;
                that[maxSelector] = isDate ? new Date(year, 11, 31) : 10
            }
            that.stubData = true;
            return that
        },
        correctValueZeroLevel: function() {
            var that = this;
            if ("logarithmic" === that[axisTypeSelector] || _isDate(that[maxSelector]) || _isDate(that[minSelector])) {
                return that
            }

            function setZeroLevel(min, max) {
                that[min] < 0 && that[max] < 0 && (that[max] = 0);
                that[min] > 0 && that[max] > 0 && (that[min] = 0)
            }
            setZeroLevel(minSelector, maxSelector);
            setZeroLevel(minVisibleSelector, maxVisibleSelector);
            return that
        },
        checkZeroStick: function() {
            var that = this;
            if (that.min >= 0 && that.max >= 0) {
                that.minStickValue = 0
            } else {
                if (that.min <= 0 && that.max <= 0) {
                    that.maxStickValue = 0
                }
            }
            return that
        }
    }
});
