/** 
 * DevExtreme (viz/tree_map/hover.js)
 * Version: 16.1.4
 * Build date: Wed Jun 22 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Web
 */
"use strict";
define(function(require, exports, module) {
    var proto = require("./tree_map.base").prototype,
        nodeProto = require("./node").prototype,
        common = require("./common"),
        _parseScalar = require("../core/utils").parseScalar,
        _buildRectAppearance = common.buildRectAppearance,
        CHANGE_HOVER_ENABLED = common.addChangeCode("CHANGE_HOVER_ENABLED"),
        STATE_CODE = 1;
    require("./api");
    require("./states");
    proto._eventsMap.onHoverChanged = {
        name: "hoverChanged"
    };
    proto._themeDependentChanges |= CHANGE_HOVER_ENABLED;
    common.expand(proto._handlers, "calculateAdditionalStates", function(states, options) {
        states[1] = options.hoverStyle ? _buildRectAppearance(options.hoverStyle) : {}
    });
    nodeProto.statesMap[1] = 1;
    nodeProto.additionalStates.push(1);
    common.expand(proto, "_extendProxyType", function(proto) {
        var that = this;
        proto.setHover = function() {
            that._hoverNode(this._id)
        };
        proto.isHovered = function() {
            return that._hoverIndex === this._id
        }
    });
    common.expand(proto, "_onNodesCreated", function() {
        this._hoverIndex = -1
    });
    common.expand(proto, "_handleChangedOptionsCore", function(options) {
        if ("hoverEnabled" in options) {
            this._change(CHANGE_HOVER_ENABLED)
        }
    });
    common.expand(proto, "_applyLogicalChanges", function() {
        if (this._changes & CHANGE_HOVER_ENABLED) {
            this._changeHoverEnabled()
        }
    });
    proto._changeHoverEnabled = function() {
        var hoverEnabled = _parseScalar(this._getOption("hoverEnabled", true), true);
        if (!hoverEnabled) {
            this.clearHover()
        }
        this._hoverEnabled = hoverEnabled
    };
    proto._applyHoverState = function(state) {
        var node = this._hoverIndex >= 0 && this._nodes[this._hoverIndex];
        if (node) {
            setNodeStateRecursive(node, STATE_CODE, state);
            this._eventTrigger("hoverChanged", {
                node: node.proxy
            })
        }
    };

    function setNodeStateRecursive(node, code, state) {
        var i, nodes = node.isNode() && node.nodes,
            ii = nodes && nodes.length;
        node.setState(code, state);
        for (i = 0; i < ii; ++i) {
            setNodeStateRecursive(nodes[i], code, state)
        }
    }
    proto._hoverNode = function(index) {
        var that = this;
        if (that._hoverEnabled && that._hoverIndex !== index) {
            that._context.suspend();
            that._applyHoverState(false);
            that._hoverIndex = index;
            if (index >= 0) {
                that._applyHoverState(true)
            }
            that._context.resume()
        }
    };
    proto.clearHover = function() {
        this._hoverNode(-1)
    }
});
